# Benchmarks

## Sample apps

All apps employ two global middlewares with `req` mutations, an empty `GET` route handler for `favicon.ico` and a GET handler for the `/users/:id`, returning a `User: {id}` string response.

## Benchmarking command

Results are taken after one run. The command used for results is the following:

```sh
node run.mjs
```

`run.mjs` is a script that launches a child process with the app and runs benchmarks against it, recursively, going to the next app in the list.

## Information

### Hardware

Xiaomi Laptop with 16GB RAM and Intel Core i7-8550U processor.

### Software

- **OS**: Manjaro Linux
- **Kernel version**: 5.7.19-2-MANJARO
- **Node.js**: 15.2.1

## Results

> **Note**: benchmarks aren't completely accurate and are different on every run and on every machine. You need to compare proportions instead of absolute values.

The table takes average results.

| framework                  | req/s | transfer/sec | latency |
| -------------------------- | ----- | ------------ | ------- |
| @tinyhttp/app (w/o exts)   | 24575 | 3.81 MB      | 3.37 ms |
| @tinyhttp/app (esm)        | 22820 | 3.54 MB      | 4.04 ms |
| @tinyhttp/app (cjs)        | 22637 | 3.51 MB      | 4.08 ms |
| express@4.17.1             | 12986 | 2 MB         | 7.11 ms |
| polka@0.5.2                | 36078 | 4.73 MB      | 2.25 ms |
| koa@2.13 + @koa/router@9.4 | 20790 | 3.58 MB      | 4.24 ms |

**Conclusion**: tinyhttp (without extra `req` / `res` extensions) is ~1.9x faster than Express and ~1.5x slower than Polka.

## Detailed results

- tinyhttp (without extra `req` / `res` extensions)

```
┌─────────┬──────┬──────┬───────┬──────┬─────────┬─────────┬───────┐
│ Stat    │ 2.5% │ 50%  │ 97.5% │ 99%  │ Avg     │ Stdev   │ Max   │
├─────────┼──────┼──────┼───────┼──────┼─────────┼─────────┼───────┤
│ Latency │ 3 ms │ 3 ms │ 5 ms  │ 7 ms │ 3.37 ms │ 1.47 ms │ 92 ms │
└─────────┴──────┴──────┴───────┴──────┴─────────┴─────────┴───────┘
┌───────────┬────────┬────────┬─────────┬─────────┬──────────┬─────────┬────────┐
│ Stat      │ 1%     │ 2.5%   │ 50%     │ 97.5%   │ Avg      │ Stdev   │ Min    │
├───────────┼────────┼────────┼─────────┼─────────┼──────────┼─────────┼────────┤
│ Req/Sec   │ 14863  │ 14863  │ 25423   │ 25695   │ 24575.74 │ 2628.68 │ 14857  │
├───────────┼────────┼────────┼─────────┼─────────┼──────────┼─────────┼────────┤
│ Bytes/Sec │ 2.3 MB │ 2.3 MB │ 3.94 MB │ 3.98 MB │ 3.81 MB  │ 407 kB  │ 2.3 MB │
└───────────┴────────┴────────┴─────────┴─────────┴──────────┴─────────┴────────┘

Req/Bytes counts sampled once per second.

369k requests in 15.04s, 57.1 MB read
41 errors (0 timeouts)
```

- tinyhttp (esm)

```
┌─────────┬──────┬──────┬───────┬──────┬─────────┬─────────┬───────┐
│ Stat    │ 2.5% │ 50%  │ 97.5% │ 99%  │ Avg     │ Stdev   │ Max   │
├─────────┼──────┼──────┼───────┼──────┼─────────┼─────────┼───────┤
│ Latency │ 3 ms │ 4 ms │ 6 ms  │ 8 ms │ 4.04 ms │ 1.36 ms │ 69 ms │
└─────────┴──────┴──────┴───────┴──────┴─────────┴─────────┴───────┘
┌───────────┬─────────┬─────────┬─────────┬─────────┬──────────┬─────────┬─────────┐
│ Stat      │ 1%      │ 2.5%    │ 50%     │ 97.5%   │ Avg      │ Stdev   │ Min     │
├───────────┼─────────┼─────────┼─────────┼─────────┼──────────┼─────────┼─────────┤
│ Req/Sec   │ 11447   │ 11447   │ 23567   │ 24383   │ 22820.54 │ 3098.67 │ 11442   │
├───────────┼─────────┼─────────┼─────────┼─────────┼──────────┼─────────┼─────────┤
│ Bytes/Sec │ 1.77 MB │ 1.77 MB │ 3.65 MB │ 3.78 MB │ 3.54 MB  │ 481 kB  │ 1.77 MB │
└───────────┴─────────┴─────────┴─────────┴─────────┴──────────┴─────────┴─────────┘

Req/Bytes counts sampled once per second.

342k requests in 15.02s, 53.1 MB read
495 errors (0 timeouts)
```

- tinyhttp (cjs)

```
┌─────────┬──────┬──────┬───────┬──────┬─────────┬─────────┬───────┐
│ Stat    │ 2.5% │ 50%  │ 97.5% │ 99%  │ Avg     │ Stdev   │ Max   │
├─────────┼──────┼──────┼───────┼──────┼─────────┼─────────┼───────┤
│ Latency │ 3 ms │ 4 ms │ 5 ms  │ 7 ms │ 4.08 ms │ 1.37 ms │ 69 ms │
└─────────┴──────┴──────┴───────┴──────┴─────────┴─────────┴───────┘
┌───────────┬─────────┬─────────┬─────────┬─────────┬─────────┬────────┬─────────┐
│ Stat      │ 1%      │ 2.5%    │ 50%     │ 97.5%   │ Avg     │ Stdev  │ Min     │
├───────────┼─────────┼─────────┼─────────┼─────────┼─────────┼────────┼─────────┤
│ Req/Sec   │ 12847   │ 12847   │ 23455   │ 23823   │ 22637.6 │ 2667.1 │ 12843   │
├───────────┼─────────┼─────────┼─────────┼─────────┼─────────┼────────┼─────────┤
│ Bytes/Sec │ 1.99 MB │ 1.99 MB │ 3.64 MB │ 3.69 MB │ 3.51 MB │ 413 kB │ 1.99 MB │
└───────────┴─────────┴─────────┴─────────┴─────────┴─────────┴────────┴─────────┘

Req/Bytes counts sampled once per second.

340k requests in 15.02s, 52.6 MB read
336 errors (0 timeouts)
```

- express@4.17.1

```
┌─────────┬──────┬──────┬───────┬───────┬─────────┬─────────┬───────┐
│ Stat    │ 2.5% │ 50%  │ 97.5% │ 99%   │ Avg     │ Stdev   │ Max   │
├─────────┼──────┼──────┼───────┼───────┼─────────┼─────────┼───────┤
│ Latency │ 6 ms │ 7 ms │ 11 ms │ 15 ms │ 7.11 ms │ 2.51 ms │ 98 ms │
└─────────┴──────┴──────┴───────┴───────┴─────────┴─────────┴───────┘
┌───────────┬────────┬────────┬─────────┬─────────┬──────────┬─────────┬────────┐
│ Stat      │ 1%     │ 2.5%   │ 50%     │ 97.5%   │ Avg      │ Stdev   │ Min    │
├───────────┼────────┼────────┼─────────┼─────────┼──────────┼─────────┼────────┤
│ Req/Sec   │ 6407   │ 6407   │ 13559   │ 13711   │ 12986.54 │ 1789.57 │ 6405   │
├───────────┼────────┼────────┼─────────┼─────────┼──────────┼─────────┼────────┤
│ Bytes/Sec │ 987 kB │ 987 kB │ 2.09 MB │ 2.11 MB │ 2 MB     │ 276 kB  │ 986 kB │
└───────────┴────────┴────────┴─────────┴─────────┴──────────┴─────────┴────────┘

Req/Bytes counts sampled once per second.

195k requests in 15.01s, 30 MB read
700 errors (0 timeouts)
```

- polka@0.5.2

```
┌─────────┬──────┬──────┬───────┬──────┬─────────┬─────────┬───────┐
│ Stat    │ 2.5% │ 50%  │ 97.5% │ 99%  │ Avg     │ Stdev   │ Max   │
├─────────┼──────┼──────┼───────┼──────┼─────────┼─────────┼───────┤
│ Latency │ 2 ms │ 2 ms │ 4 ms  │ 5 ms │ 2.25 ms │ 1.36 ms │ 89 ms │
└─────────┴──────┴──────┴───────┴──────┴─────────┴─────────┴───────┘
┌───────────┬─────────┬─────────┬─────────┬─────────┬──────────┬─────────┬─────────┐
│ Stat      │ 1%      │ 2.5%    │ 50%     │ 97.5%   │ Avg      │ Stdev   │ Min     │
├───────────┼─────────┼─────────┼─────────┼─────────┼──────────┼─────────┼─────────┤
│ Req/Sec   │ 24287   │ 24287   │ 38751   │ 40095   │ 36078.94 │ 5083.35 │ 24286   │
├───────────┼─────────┼─────────┼─────────┼─────────┼──────────┼─────────┼─────────┤
│ Bytes/Sec │ 3.18 MB │ 3.18 MB │ 5.07 MB │ 5.26 MB │ 4.73 MB  │ 666 kB  │ 3.18 MB │
└───────────┴─────────┴─────────┴─────────┴─────────┴──────────┴─────────┴─────────┘

Req/Bytes counts sampled once per second.

541k requests in 15.02s, 70.9 MB read
200 errors (0 timeouts)
```

- koa@2.13 + @koa/router@9.4

```
┌─────────┬──────┬──────┬───────┬───────┬─────────┬─────────┬───────┐
│ Stat    │ 2.5% │ 50%  │ 97.5% │ 99%   │ Avg     │ Stdev   │ Max   │
├─────────┼──────┼──────┼───────┼───────┼─────────┼─────────┼───────┤
│ Latency │ 3 ms │ 4 ms │ 8 ms  │ 11 ms │ 4.24 ms │ 2.29 ms │ 91 ms │
└─────────┴──────┴──────┴───────┴───────┴─────────┴─────────┴───────┘
┌───────────┬─────────┬─────────┬────────┬─────────┬──────────┬─────────┬─────────┐
│ Stat      │ 1%      │ 2.5%    │ 50%    │ 97.5%   │ Avg      │ Stdev   │ Min     │
├───────────┼─────────┼─────────┼────────┼─────────┼──────────┼─────────┼─────────┤
│ Req/Sec   │ 8943    │ 8943    │ 22111  │ 25055   │ 20790.41 │ 4158.46 │ 8941    │
├───────────┼─────────┼─────────┼────────┼─────────┼──────────┼─────────┼─────────┤
│ Bytes/Sec │ 1.54 MB │ 1.54 MB │ 3.8 MB │ 4.31 MB │ 3.58 MB  │ 715 kB  │ 1.54 MB │
└───────────┴─────────┴─────────┴────────┴─────────┴──────────┴─────────┴─────────┘

Req/Bytes counts sampled once per second.

312k requests in 15.02s, 53.6 MB read
700 errors (0 timeouts)
```
